package name.krestjaninoff.application.aspect;

/**
 * DAO Factory aspect
 * 
 * @author Mikhail Krestjaninoff
 */
public aspect DAOFactoryAspect {

    /**
     * Creating object of DAO using "new()" construction
     */
    pointcut createDAO(): call((name.krestjaninoff.application.data.*DAO).new(..) );

    /**
     * Code within getDAO method of DAOFactory class
     */
    pointcut withinFactory(): withincode(
		* name.krestjaninoff.application.data.DAOFactory.getMapper(..) );

    /**
     * Advice for calling "new()" construction for DAO objects outside DAOFactory
     */
    before() throws IllegalAccessException : createDAO() && !withinFactory() {
	throw new IllegalAccessException("DAO object can't be created"
		+ " using 'new()' construction");
    }
}