package name.krestjaninoff.application.aspect;

import org.apache.log4j.Logger;

/**
 * Logging method call
 * 
 * @author Mikhail Krestjaninoff
 */
public aspect LoggerAspect {
    final Logger log = Logger.getLogger(LoggerAspect.class);

    /**
     * Web-method call
     */
    pointcut MethodCall() : execution(@name.krestjaninoff.application.service.Test * *(..));

    /**
     * Method start
     */
    before() : MethodCall() {
	this.log.debug("Start method "
		+ thisJoinPoint.getSignature().toString());
    }

    /**
     * Method finish
     */
    after() returning: MethodCall() {
	this.log.debug("Finish method "
		+ thisJoinPoint.getSignature().toString());
    }

    /**
     * Method finish (exception thrown)
     */
    after() throwing ( Error ex): MethodCall() {
	this.log.error("Finish method "
		+ thisJoinPoint.getSignature().toString() + " with exception "
		+ ex.getMessage());
    }
}
