package name.krestjaninoff.application;

import name.krestjaninoff.application.exception.AuthException;
import name.krestjaninoff.application.interfaces.AnonymousService;

import org.apache.cxf.jaxws.JaxWsProxyFactoryBean;
import org.apache.log4j.Logger;

/**
 * Client for service version of application
 * <br /><br /> 
 * Connects to service and outputs his version
 *
 * @author Mikhail Krestjaninoff
 */
public class Client {
	
	final static String DEFAULT_URL = "http://localhost:8081/services/";
	final static String DEFAULT_LOGIN = "login";
	final static String DEFAULT_PASSWORD = "password";
	
	public static void main(String[] args) {
		
		// Set up logger
		//PropertyConfigurator.configure("/etc/mk-application/log4j.properties");
		Logger log = Logger.getLogger(Client.class);
		
        AnonymousService anService;
		try {
		    // Get credentials
		    String url = ( args.length >= 1 ) ? args[0] : DEFAULT_URL;
		    String login = ( args.length >= 2 ) ? args[1] : DEFAULT_LOGIN;
		    String password = ( args.length >= 3 ) ? args[2] : DEFAULT_PASSWORD;
		    if( login.isEmpty() || password.isEmpty() )
		    	throw new AuthException("Empty credentials");
		    
		    // Create service factory
		    JaxWsProxyFactoryBean serviceFactory = new JaxWsProxyFactoryBean();
		    serviceFactory.setUsername(login);
		    serviceFactory.setPassword(password);
		    serviceFactory.setAddress(url + "anonymous" + "/");
		    serviceFactory.setServiceClass(AnonymousService.class);

		    // Create service
		    log.info("Creation of anonymouse service client...");
		    anService = (AnonymousService) serviceFactory.create();
		    log.info("Creation of anonymouse service client completed");
		    
	        // Show version
		    log.info("Calling method \"getVersion\" of anonymouse service...");
	        System.out.println("Current version is " + anService.getVersion() );
	        log.info("Calling method \"getVersion\" of anonymouse service completed");
		    
	    // Handle errors 
        } catch( AuthException e ) {
        	System.err.println("Invalid credentials");
	    	System.exit(1);
        } catch (Exception e) {
        	System.err.println("Unknown Exception");
        	e.printStackTrace();
		}
	}
}
