package name.krestjaninoff.application;

import name.krestjaninoff.application.domain.User;
import name.krestjaninoff.application.exception.AuthException;
import name.krestjaninoff.application.interfaces.WorkflowService;

import org.apache.log4j.Logger;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;


/**
 * Console version of application
 * <br /><br />
 * Outputs info about current user
 *
 * @author Mikhail Krestjaninoff
 */
public class Console {
	public static void main(String[] args) {
		
		// Set up logger
		Logger log = Logger.getLogger(Console.class);

        // Init application context
		ApplicationContext context = new ClassPathXmlApplicationContext("application-context.xml");
        log.info("APPLICATION STARTED");
        
		WorkflowService wfService;
        try {
		    // Get credentials
		    if( args.length < 2 )
		    	throw new AuthException();
		    String login = args[0];
		    String password = args[1];

		    // Auth
		    AuthenticationManager authManager = context.getBean("authManager",
		    		AuthenticationManager.class);
		    Authentication request = new UsernamePasswordAuthenticationToken(
		    		login, password);
		    Authentication result = authManager.authenticate(request);
		    SecurityContextHolder.getContext().setAuthentication(result);
		    
		    // Get user info
		    wfService = context.getBean("workflowService", WorkflowService.class);
		    User user = wfService.getUser();
		    
	        // Show version
	        System.out.println("User info: " + user.getFullName() );
		    
        } catch( AuthException e ) {
        	System.err.println("Invalid credentials");
	    	System.exit(1);
        }
	}
}
