package name.krestjaninoff.application.domain;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * Order DTO
 *
 * @author Mikhail Krestjaninoff
 */
@Entity
@Table(name="order")
@SequenceGenerator(
		name="sequenceOrder",
		sequenceName="order_id_seq"
)
@Cache(usage=CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("unused")
public class Order implements Serializable {
	private static final long serialVersionUID = 84504362507297200L;
	
	/**
	 * Identifier
	 */
	private Long id;
	
	/**
	 * Title
	 */
	private String title;
	
	/**
	 * Description
	 */
	private String description;
	
	/**
	 * Price
	 */
	private Double price;
	
	/**
	 * Creation Date
	 */
	private Date date;
	
	/**
	 * Customer of order
	 */
	private User user;
	

	public Order() {
		this.date = Calendar.getInstance().getTime();
	}
	
	
	@Id @GeneratedValue(strategy=GenerationType.SEQUENCE, generator="sequenceOrder")
	public Long getId() {
	    return this.id;
	}
	private void setId(Long id) {
	    this.id = id;
	}
	
	@Column(name="title", nullable=false, length=255)
	public String getTitle() {
	    return this.title;
	}
	public void setTitle(String title) {
	    this.title = title;
	}
	
	@Column(name="description", nullable=true )
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}

	@Column(name="price", nullable=false, precision=2)
	public Double getPrice() {
		return price;
	}
	public void setPrice(Double price) {
		this.price = price;
	}

	@Column(name="date", nullable=false)
	public Date getDate() {
		return date;
	}
	public void setDate(Date date) {
		this.date = date;
	}
	
	@ManyToOne
	@JoinColumn(name="user_id")
	public User getUser(){
		return user;
	}
	public void setUser(User user){
		this.user = user;
	}
	
	@Override
	public String toString() {
		String object = "Id: " + this.getId() + ", Name: " + getTitle();
		return object; 
	}
}
