package name.krestjaninoff.application.domain;

import java.io.Serializable;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.SequenceGenerator;
import javax.security.auth.login.LoginException;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * User DTO
 *
 * @author Mikhail Krestjaninoff
 */
@Entity
@Table(name="user")
@SequenceGenerator(
		name="sequenceUser",
		sequenceName="user_id_seq"
)
@Cache(usage=CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings("unused")
public class User implements Serializable {
	private static final long serialVersionUID = 8372128484215085291L;

	/**
	 * Identifier
	 */
	private Long id;
	
	/**
	 * FirstName
	 */
	private String firstName;
	
	/**
	 * MiddleName
	 */
	private String middleName;
	
	/**
	 * SurName
	 */
	private String surName;
	
	/**
	 * Login
	 */
	private String login;
	
	/**
	 * Password
	 */
	private String password;
	
	/**
	 * Set of user's orders
	 */
	private Set<Order> orders;
	
	
	public User() {}

	
	@Id @GeneratedValue(strategy=GenerationType.SEQUENCE, generator="sequenceUser")
	public Long getId() {
	    return this.id;
	}
	private void setId(Long id) {
	    this.id = id;
	}

	@Column(name="firstName", nullable=false, length=255)
	public String getFirstName() {
		return firstName;
	}
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	@Column(name="middleName", nullable=true, length=255)
	public String getMiddleName() {
		return middleName;
	}
	public void setMiddleName(String middleName) {
		this.middleName = middleName;
	}

	@Column(name="surName", nullable=false, length=255)
	public String getSurName() {
		return surName;
	}
	public void setSurName(String surName) {
		this.surName = surName;
	}
	
	@Transient
	public String getFullName() {
		return this.firstName + " " + this.surName;
	}

	@Column(name="login", nullable=false, length=255, unique=true)
	public String getLogin() {
		return login;
	}
	public void setLogin(String login) {
		this.login = login;
	}

	@Column(name="password", nullable=false, length=255)
	public String getPassword() {
		return password;
	}
	public void setPassword(String password) {
		this.password = password;
	}
	
	@OneToMany(mappedBy="user", cascade=CascadeType.PERSIST, fetch=FetchType.EAGER)
	public Set<Order> getOrders() {
		return orders;
	}
	public void setOrders(Set<Order> orders) {
		this.orders = orders;
	}
	
	@Override
	public String toString() {
		String object = "Id: " + this.getId() + ", Name: " + getFullName();
		return object; 
	}
}
