package name.krestjaninoff.application.domain;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import name.krestjaninoff.application.exception.LogicalException;

import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

/**
 * User DAO 
 *
 * @author Mikhail Krestjaninoff
 */
@Component("userDAO")
public class UserDAO {

	@PersistenceContext
	private EntityManager entityManager;	
	
	/**
	 * Get user by login
	 * 
	 * @param login User's login
	 * @return user
	 * @throws LogicalException 
	 */
	@Transactional
	public User getByLogin(String login) throws LogicalException {

		List<?> users = entityManager
			.createQuery(
					"SELECT user FROM User as user"
							+ " WHERE user.login = :login")
			.setParameter("login", login)
			.getResultList();
		
		if (users.isEmpty())
			throw new LogicalException("User with login " + login + " not found");
		User user = (User) users.iterator().next();
		
		return user;
	}
}
