package name.krestjaninoff.application.interceptor;

import name.krestjaninoff.application.domain.User;
import name.krestjaninoff.application.domain.UserDAO;
import name.krestjaninoff.application.service.WorkflowServiceImpl;

import org.apache.log4j.Logger;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;

/**
 * Interceptor for binding current user to service 
 *
 * @author Mikhail Krestjaninoff
 */
@Aspect
@Component
@Order(100)
public class AuthInterceptor {
	private static Logger log = Logger.getLogger(AuthInterceptor.class);

	@Autowired
	private UserDAO userDAO;
	
	/**
	 * Invocation handler
	 * 
	 * @param methodInvocation Invoked method
	 * @throws Throwable
	 */
	@Around("* name.krestjaninoff.applicarion.service.WorkflowServiceImpl.*(..)")
	public Object invoke(ProceedingJoinPoint joinPoint) throws Throwable {
		
		Authentication auth = SecurityContextHolder.getContext().getAuthentication(); 
		String login = auth.getName();
		log.info("Binding auth " + auth + " of user " + login);
		
		User user = userDAO.getByLogin(login);
		((WorkflowServiceImpl) joinPoint.getTarget()).setUser(user);
		
		Object result = joinPoint.proceed( joinPoint.getArgs() );
		return result;
	}
}
