package name.krestjaninoff.application.interceptor;

import java.io.Serializable;

import javax.annotation.Resource;

import name.krestjaninoff.application.util.Algorithm;
import net.sf.ehcache.Cache;
import net.sf.ehcache.Element;

import org.apache.log4j.Logger;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;


/**
 * Iterceptor for caching method call
 * 
 * @author Mikhail Krestjaninoff
 */
@Aspect
@Component
@Order(200)
public class CacheInterceptor {
	private static Logger log = Logger.getLogger(CacheInterceptor.class);
	
	@Resource(name="cacheCommon")
	private Cache cache;
	
	
	@Around("@annotation(name.krestjaninoff.application.annotation.Cached)")
	public Object invoke(ProceedingJoinPoint joinPoint) throws Throwable {
		
		// Init variables
		String methodName = joinPoint.getSignature().toString();
		Object[] args = joinPoint.getArgs();
		Object result = null;

		// Retrieve method parameters
		StringBuilder sArgs = new StringBuilder();
		for (int i = 0; i < args.length; i++) {
			if (i != 0) {
				sArgs.append(", ");
			}
			sArgs.append(args[i] != null ? args[i].getClass().getName() + " "
					+ args[i] : "null");
		}

		// Create cache key
		final String cacheKey = Algorithm.md5( methodName + ":" + sArgs.toString() );
		
		// Get cached data by key
		final Element cacheElement = cache.get( cacheKey );
		
		// Cached data not found
		if( null == cacheElement ) {
			log.debug("Cache [ key:" + cacheKey + ", value: not found]");
			
			// Call method 
			result = joinPoint.proceed( args );
			
			// Save data in cache
			final Element newElement = new Element( cacheKey,
					(Serializable) result);
			cache.put(newElement);
		
		// Cached data found 
		} else {
			log.debug("Cache [ key:" + cacheKey + ", value: found]");
			
			result = cacheElement.getValue();
		}
				
		return result;
	}
}
