package name.krestjaninoff.application.interceptor;

import java.lang.reflect.Method;
import java.math.BigDecimal;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.lang.time.DateUtils;
import org.apache.commons.lang.time.StopWatch;
import org.apache.log4j.Logger;

/**
 * Interceptor for logging methods call
 * 
 * @author Mikhail Krestjaninoff
 */
public class LogInterceptor implements MethodInterceptor {
	private static Logger log = Logger.getLogger(LogInterceptor.class);

	/**
	 * Invocation handler
	 * 
	 * @param methodInvocation Invoked method
	 * @throws Throwable
	 */
	public Object invoke(MethodInvocation methodInvocation) throws Throwable {
		
		// Init variables
		Method method = methodInvocation.getMethod();
		Object[] args = methodInvocation.getArguments();

		// Retrieve method name
		String methodName = method.getDeclaringClass().getName() + "."
				+ method.getName();

		// Retrieve method parameters
		StringBuilder sArgs = new StringBuilder();
		for (int i = 0; i < args.length; i++) {
			if (i != 0) {
				sArgs.append(", ");
			}
			sArgs.append(args[i] != null ? args[i].getClass().getName() + " "
					+ args[i] : "null");
		}
		
		// Start timer
		final StopWatch stopWatch = new StopWatch();
		stopWatch.start();
		
		// Method call
		log.info("Call method: " + methodName + "(" + sArgs.toString() + ") ");
		Object result = methodInvocation.proceed();
		
		// Stop timer
		final long millis = stopWatch.getTime();
		final BigDecimal seconds = new BigDecimal(millis).divide(
			      new BigDecimal(DateUtils.MILLIS_PER_SECOND), BigDecimal.ROUND_HALF_UP);
		
		// Log results
		log.info("Method " + methodName + " executed in " +
				seconds + " seconds (" + millis + " millis)" +
				" and return: " + result );

		return result;
	}
}
