package name.krestjaninoff.application.logic;

import java.util.Set;

import org.springframework.stereotype.Component;

import name.krestjaninoff.application.domain.Order;
import name.krestjaninoff.application.domain.User;

/**
 * Discount domain
 * 
 * @author Mikhail Krestjaninoff
 */
@Component
public class DiscountLogic {

	public DiscountLogic() {
	}

	/**
	 * Calculate user's discount
	 * 
	 * @param user
	 *            Customer
	 * @return User's discount
	 */
	public Double getDiscount(User user) {
		Double discount = 0.;
		Double ordersSum = this.getOrdersSum(user);

		if (ordersSum > 100)
			discount = 10.;

		return discount;
	}

	/**
	 * Calculate user's orders sum
	 * 
	 * @param user
	 *            Customer
	 * @return User's orders sum
	 */
	private Double getOrdersSum(User user) {
		Double ordersSum = 0.;
		Set<Order> orders = user.getOrders();

		for (Order theOrder : orders) {
			ordersSum += theOrder.getPrice();
		}

		return ordersSum;
	}
}
