package name.krestjaninoff.application.security;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.stereotype.Component;

/**
 * Authentication Manager
 * 
 * @author Mikhail Krestjaninoff
 */
@Component("authManager")
public class AuthenticationManagerImpl implements AuthenticationManager {
	private static final long serialVersionUID = -7959241106661007462L;

    static final List<GrantedAuthority> AUTHORITIES = new ArrayList<GrantedAuthority>();
    static {
      AUTHORITIES.add(new GrantedAuthorityImpl("ROLE_USER"));
    }
	
	@Resource(name="userDetailService")
	private UserDetailsService userDetailService;
	

	/**
	 * Check password
	 * 
	 * @param userPassword
	 * @param originalPassword
	 * @return
	 */
	public static boolean isPasswordValid(String userPassword, String originalPassword) {
		boolean isValid = originalPassword.equals(userPassword);
		return isValid;
	}
	
	@Override
	public Authentication authenticate(Authentication authentication)
			throws AuthenticationServiceException {
		String name = authentication.getName();
		
		// Get user details
		UserDetails userDetails = userDetailService.loadUserByUsername(name);
		if (null == userDetails) {
			 throw new AuthenticationServiceException("User with login " + name +
					 " not found");
		}

		// Check user password
		String originalPassword = userDetails.getPassword();
		String userPassword = (String) authentication.getCredentials();
		if (isPasswordValid(userPassword, originalPassword)) {
			authentication = new UsernamePasswordAuthenticationToken(
					authentication.getName(), authentication.getCredentials(),
					AUTHORITIES);
		} else {
			throw new BadCredentialsException("Invalid credentials for login "
					+ name);
		}
		
		return authentication;
	}
}