package name.krestjaninoff.application.security;

import java.util.ArrayList;
import java.util.Collection;

import name.krestjaninoff.application.domain.User;
import name.krestjaninoff.application.domain.UserDAO;
import name.krestjaninoff.application.exception.LogicalException;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

/**
 * Provider of UserDetails for Spring Security authentication 
 *
 * @author Mikhail Krestjaninoff
 */
@Component("userDetailsService")
public class UserDetailsServiceImpl implements UserDetailsService {
	private static Logger log = Logger.getLogger(UserDetailsServiceImpl.class);

	@Autowired
	private UserDAO userDAO;
	
	@Transactional
	@Override
	public UserDetails loadUserByUsername(String arg0)
			throws UsernameNotFoundException, DataAccessException {
		UserDetailsImpl userDetail;
		
		log.info("Read info about user with login " + arg0);
		try {
			// Read user data
			User user;
			try {
				user = userDAO.getByLogin(arg0);
			} catch (LogicalException e) {
				throw new UsernameNotFoundException("User with login " +
						arg0 + " not found", e);
			}
			
			// Set user details
			userDetail = new UserDetailsImpl();
			
			userDetail.setUsername(user.getLogin());
			userDetail.setPassword(user.getPassword());
			
			// Set roles (can be read from db) 
			Collection<GrantedAuthority> authorities = new ArrayList<GrantedAuthority>(); 
			authorities.add(new GrantedAuthorityImpl("ROLE_USER"));
			userDetail.setAuthorities(authorities);
			
		} catch (Exception e) {
			throw new DataAccessEcxeptionImpl("Data access error", e);
		}
		
		log.info("Created user details: " + userDetail);
		return userDetail;
	}
}