package name.krestjaninoff.application.service;

import java.util.Set;

import javax.jws.WebService;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import name.krestjaninoff.application.domain.Order;
import name.krestjaninoff.application.domain.User;
import name.krestjaninoff.application.interfaces.WorkflowService;

import org.apache.log4j.Logger;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


/**
 * Application workflow service
 * 
 * @author Mikhail Krestjaninoff
 */
@Service("workflowService")
@WebService(
	serviceName = "WorkflowService",
	targetNamespace = "http://application.krestjaninoff.name/",
	endpointInterface = "name.krestjaninoff.application.interfaces.WorkflowService"
)
public class WorkflowServiceImpl implements WorkflowService {
	final Logger log = Logger.getLogger(WorkflowServiceImpl.class);

	@PersistenceContext
	private EntityManager entityManager;
	
	/**
	 * Current user (sets by AuthInterceptor)
	 */
	private final ThreadLocal<User> user = new ThreadLocal<User>();
	public void setUser(User user) {
		this.user.set(user);
	}
	
	
	/**
	 * Constructor
	 */
	public WorkflowServiceImpl() {
	}
	
	
	/**
	 * Get user
	 */
	@Transactional(readOnly=true)
	public User getUser() {
		return this.user.get();
	}
	
	/**
	 * Create order
	 */
	@Transactional
	public Long createOrder(Order order) {
		order.setUser(this.getUser());
		
		entityManager.persist(order);
		entityManager.flush();
		
		return order.getId();
	}

	/**
	 * Get list of orders
	 */
	@Transactional(readOnly=true)
	public Set<Order> getOrders() {
		Set<Order> orders = this.getUser().getOrders();

		return orders;
	}
	
	/**
	 * Check user acceess
	 */
	@Secured({"ROLE_USER"})
	public String checkUserAcceess() {
		return "You have user access";
	}
	
	/**
	 * Check admin acceess
	 */
	@Secured({"ROLE_ADMIN"})
	public String checkAdminAcceess() {
		return "You have user access";
	}
}
