package name.krestjaninoff.application.unused.service;

import java.util.List;

import javax.jws.WebService;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import name.krestjaninoff.application.domain.User;
import name.krestjaninoff.application.exception.AuthException;
import name.krestjaninoff.application.exception.SystemException;
import name.krestjaninoff.application.unused.session.SessionManager;

/**
 * Workflow service factory
 * 
 * @author Mikhail Krestjaninoff
 */
//@Service("authService")
@WebService(
	serviceName = "AuthService",
	targetNamespace = "http://application.krestjaninoff.name/",
	endpointInterface = "name.krestjaninoff.application.interfaces.AuthService"
)
public class AuthServiceImpl implements AuthService {
	final Logger log = Logger.getLogger(AuthServiceImpl.class);

	@PersistenceContext
	EntityManager entityManager;

	@Autowired
	SessionManager sessionManager;
	

	/**
	 * Constructor
	 */
	public AuthServiceImpl() {
	}

	
	/**
	 * Login
	 * 
	 * @throws AuthException
	 */
	public String login(String login, String password)
			throws AuthException {

		// Get user by credentials
		User user = getUser(login, password);

		// Create user's session
		String sessionId;
		try {
			sessionId = sessionManager.save(user);
		} catch (SystemException e) {
			throw new AuthException("Session creating error", e);
		}
		
		return sessionId;
	}

	/**
	 * Delete workflow service by user's login
	 * 
	 * @throws AuthException
	 */
	public void logout(String sessionId)
			throws AuthException {
		sessionManager.remove(sessionId);
	}
		
	/**
	 * Get user by credentials
	 * 
	 * @param login
	 * @param password
	 * @return User
	 * @throws AuthException
	 */
	private User getUser(String login, String password) throws AuthException {
		List<?> users = entityManager
				.createQuery(
						"select user from User as user"
								+ " where user.login = :login"
								+ " and user.password = :password")
				.setParameter("login", login)
				.setParameter("password", password)
				.getResultList();
		if (users.isEmpty())
			throw new AuthException("Invalid credentials");
		
		User user = (User) users.iterator().next();
		return user;
	}
}
