package name.krestjaninoff.application.util;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.SecureRandom;
import java.security.NoSuchAlgorithmException;

import name.krestjaninoff.application.exception.SystemException;


/**
 * Algorithm
 * 
 * @author Mikhail Krestjaninoff
 */
public class Algorithm {

	/**
	 * Create ND5 hash
	 * 
	 * @param source
	 *            Original text
	 * @return MD5 hash
	 */
	public static String md5(String source) throws Exception {
		String hash = null;

		byte[] bytesOfMessage = source.getBytes("UTF-8");
		MessageDigest md = MessageDigest.getInstance("MD5");
		byte[] digest = md.digest(bytesOfMessage);

		BigInteger bigInt = new BigInteger(1, digest);
		hash = bigInt.toString(16);
		while (hash.length() < 32)
			hash = "0" + hash;

		return hash;
	}

	/**
	 * Generate unique Id
	 * 
	 * @return Unique Id
	 * @throws LogicalException
	 */
	public static String uniqId() throws SystemException {
		StringBuilder result = new StringBuilder();

		try {
			SecureRandom prng = SecureRandom.getInstance("SHA1PRNG");
			String randomNum = new Integer(prng.nextInt()).toString();

			MessageDigest sha = MessageDigest.getInstance("SHA-1");
			byte[] digest = sha.digest(randomNum.getBytes());
			char[] digits = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
					'a', 'b', 'c', 'd', 'e', 'f' };
			for (int idx = 0; idx < digest.length; ++idx) {
				byte b = digest[idx];
				result.append(digits[(b & 0xf0) >> 4]);
				result.append(digits[b & 0x0f]);
			}
		} catch (NoSuchAlgorithmException e) {
			throw new SystemException("Error during generation unique Id", e);
		}

		return result.toString();
	}
}
