package name.krestjaninoff.application.web;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.cxf.BusException;
import org.apache.cxf.BusFactory;
import org.apache.cxf.bus.spring.SpringBusFactory;
import org.apache.cxf.resource.ResourceManager;
import org.apache.cxf.transport.DestinationFactory;
import org.apache.cxf.transport.DestinationFactoryManager;
import org.apache.cxf.transport.servlet.CXFServlet;
import org.apache.cxf.transport.servlet.ServletContextResourceResolver;
import org.apache.cxf.transport.servlet.ServletController;
import org.apache.cxf.transport.servlet.ServletTransportFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.GenericWebApplicationContext;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import java.util.Enumeration;

/**
 * Custom CXF servlet dispatcher
 * 
 * @see http://ayax79.wordpress.com/2009/02/19/making-spring-mvc-and-cxf-play-well-together/
 */
public class CXFDispatcherServlet extends CXFServlet implements 
		ApplicationContextAware {
	private static final long serialVersionUID = -3252602650204695744L;

	private static final Log log = LogFactory
			.getLog(CXFDispatcherServlet.class);

	protected WebApplicationContext applicationContext;

	public void setApplicationContext(ApplicationContext applicationContext)
			throws BeansException {
		GenericWebApplicationContext wac = new GenericWebApplicationContext();
		
        wac.setParent(applicationContext);
        wac.refresh();
		
		this.applicationContext = wac;
	}

	@Override
	public void handleRequest(HttpServletRequest request,
			HttpServletResponse response) {
		try {
			BusFactory.setThreadDefaultBus(bus);
			controller.invoke(request, response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		} finally {
			BusFactory.setThreadDefaultBus(null);
		}
	}

	@Override
	public void init() {
		bus = new SpringBusFactory(applicationContext).createBus();

		ResourceManager resourceManager = bus
				.getExtension(ResourceManager.class);
		resourceManager.addResourceResolver(new ServletContextResourceResolver(
				applicationContext.getServletContext()));

		replaceDestinationFactory();

		controller = new ServletController(servletTransportFactory,
				SERVLET_CONFIG, applicationContext.getServletContext(), bus);
	}

	@Override
	public void destroy() {
		bus.shutdown(true);
	}

	@Override
	protected void replaceDestinationFactory() {
		DestinationFactoryManager dfm = bus
				.getExtension(DestinationFactoryManager.class);
		try {
			DestinationFactory df = dfm
					.getDestinationFactory("http://cxf.apache.org/transports/http/configuration");
			if (df instanceof ServletTransportFactory) {
				servletTransportFactory = (ServletTransportFactory) df;
				log.info("DESTIONFACTORY_ALREADY_REGISTERED");
				return;
			}
		} catch (BusException e) {
			log.info("UNKNOWN_BUS_EXCEPTION");
		}

		DestinationFactory factory = createServletTransportFactory();

		for (String s : factory.getTransportIds()) {
			registerTransport(factory, s);
		}
		log.info("REPLACED_HTTP_DESTIONFACTORY");
	}

	private void registerTransport(DestinationFactory factory, String namespace) {
		bus.getExtension(DestinationFactoryManager.class)
				.registerDestinationFactory(namespace, factory);
	}

	@Override
	protected DestinationFactory createServletTransportFactory() {
		if (servletTransportFactory == null) {
			servletTransportFactory = new ServletTransportFactory(bus);
		}
		return servletTransportFactory;
	}

	/**
	 * Annoying hack
	 * 
	 * The ServletController class uses the servlet config to get other init
	 * parameters. This mostly just prevents ServletController from throwing
	 * NullPointerExceptions
	 * 
	 * We don't care about any of them but here is teh list if we decide we do:
	 * hide-service-list-page disable-address-updates base-address
	 * service-list-stylesheet
	 */
	public static ServletConfig SERVLET_CONFIG = new ServletConfig() {

		public String getServletName() {
			return null;
		}

		public ServletContext getServletContext() {
			return null;
		}

		public String getInitParameter(String name) {
			return null;
		}

		public Enumeration<?> getInitParameterNames() {
			return null;
		}
	};
}
