package name.krestjaninoff.application.web;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

/**
 * Example of servlet request filter
 */
public class SimpleFilter implements Filter {
	private static Logger logger = Logger.getLogger(SimpleFilter.class);

	private final ThreadLocal<String> requestId = new ThreadLocal<String>();

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy() {
		requestId.remove();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest,
	 * javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws ServletException, IOException {

		try {
			HttpServletRequest req = (HttpServletRequest) request;
			String uid = req.getHeader("REQUEST_ID");
			requestId.set(uid);
			if (uid != null) {
				logger.info("SimpleFilter request: "
						+ requestId.get());
			}
			chain.doFilter(request, response);
		} finally {
			HttpServletResponse res = (HttpServletResponse) response;
			res.setHeader("REQUEST_ID", requestId.get());

			if (requestId.get() != null) {
				logger.info("SimpleFilter response: "
						+ requestId.get());
			}
		}

	}

	public String getId() {
		return requestId.get();
	}

	public void init(FilterConfig filterConfig) throws ServletException {
	}
}