package name.krestjaninoff.application.service;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertEquals;

import java.util.Set;

import javax.annotation.Resource;

import org.junit.Before;
import org.junit.Test;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.transaction.annotation.Transactional;

import name.krestjaninoff.application.AbstractITest;
import name.krestjaninoff.application.domain.Order;
import name.krestjaninoff.application.interfaces.AnonymousService;
import name.krestjaninoff.application.interfaces.WorkflowService;


/**
 * Integration test for workflow service
 *
 * @author Mikhail Krestjaninoff
 */
public class WorkflowServiceImplITest extends AbstractITest {
	
	@Resource(name="authManager")
	private AuthenticationManager authManager;
		
	@Resource(name="anonymousService")
	private AnonymousService anService;
		
	@Resource(name="workflowService")
	private WorkflowService wfService;
		
	
	@Before
	public void login() throws Throwable {
		String login = "test3";
		String password = "132";
		
	    Authentication request = new UsernamePasswordAuthenticationToken(
	    		login, password);
	    Authentication result = authManager.authenticate(request);
	    SecurityContextHolder.getContext().setAuthentication(result);
	}
	
	
	@Test
	@Transactional
	public void createOrder() {
		Order theOrder = new Order();
	    theOrder.setTitle("Test");
	    theOrder.setPrice(50.);
	    
	    Long orderId = this.wfService.createOrder(theOrder);
	    assertNotNull(orderId);
		assertTrue(orderId > 0);
	}
	
	@Test
	@Transactional
	public void getOrders() {
		Set<Order> theOrders = this.wfService.getOrders();
		
		assertNotNull(theOrders);
		assertEquals(2, theOrders.size());
	}
	
	@Test
	public void annotationCache() {
		this.anService.getVersion();
		this.anService.getVersion();
	}
}
